'' =================================================================================================
''
''   File....... jm_softtiming.spin
''   Purpose.... Software RTC/Timer that runs in own cog
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 05 MAY 2010
''   Updated.... 07 MAY 2010
''
'' =================================================================================================


CON

  #0, MD_RTC, MD_TMR                                            ' modes
  #0, REG_MS, REG_SC, REG_MN, REG_HR, REG_DY, REG_RST           ' registers


VAR
                                                                 
  long  Cog                                                     ' cog id
  
  long  Clock[6]                                                ' clock/timer registers
  long  Stack[16]                                               ' stack for Spin cog
  

PUB init(mode) | ok

  finalize                                                      ' stop if already running
  reset                                                         ' force regs reset on start
  
  case mode
    MD_RTC : ok := cognew(softrtc, @Stack) + 1                  ' run in clock mode
    MD_TMR : ok := cognew(freerun, @Stack) + 1                  ' run in raw timing mode

  return ok
  

PUB finalize

'' Stops cog (if running)

  if Cog
    cogstop(Cog~ - 1)
    longfill(@Clock, 0, 6)
    

PUB reset

'' Resets all rtc/timer registers to 0

  Clock[REG_RST]~~                                              ' -1 as flag to reset registers


PUB now(pntr, rst)

'' copies timer/rtc registers to array (5 longs) at pntr
'' -- if rst <> 0, timer/rtc is reset

  longmove(pntr, @Clock, 5)                                     ' capture clock registers
  if rst                                                        ' if reset specified
    reset                                                       '   do it!


PUB msecs

'' Returns milliseconds register from soft rtc or timer 

  return Clock[REG_MS]


PUB secs

'' Returns seconds register from soft rtc or timer 

  return Clock[REG_SC]


PUB mns

'' Returns minutes register from soft rtc or timer 

  return Clock[REG_MN]


PUB hrs

'' Returns hours register from soft rtc or timer 

  return Clock[REG_HR]


PUB days

'' Returns days register from soft rtc or timer

  return Clock[REG_DY]


PRI softrtc | ms1, t0

'' Software RTC
'' -- runs in standard "clock" mode

  ms1 := clkfreq / 1_000                                        ' ticks per millisecond
  t0  := cnt                                                    ' sync to system counter
  
  repeat
    if (Clock[REG_RST] < 0)                                     ' reset flag?
      longfill(@Clock, 0, 6)                                    ' yes, clear everything
    waitcnt(t0 += ms1)                                          ' wait 1ms
    if (++Clock[REG_MS] == 1_000)                               ' one second?
      Clock[REG_MS]~                                            ' clear ms
      if (++Clock[REG_SC] == 60)                                ' inc secs, check for 60
        Clock[REG_SC]~                                          ' clear secs
        if (++Clock[REG_MN] == 60)                              ' inc mins, check for 60
          Clock[REG_MN]~                                        ' clear mins
          if (Clock[REG_HR] == 24)                              ' in hrs, check for 24
            Clock[REG_HR]~~                                     ' clear hours
            if (Clock[REG_DY] < posx)                           ' days below max?
              ++Clock[REG_DY]                                   '  increment
            else                                                ' else
              Clock[REG_DY]~                                    '  clear


PRI freerun | ms1, t0   

'' Free-running timer
'' -- all elements (above ms) accumulate without limits
'' -- auto reset at posx seconds

  ms1 := clkfreq / 1_000                                        ' ticks per millisecond
  t0  := cnt                                                    ' sync to system counter

  repeat
    if (Clock[REG_RST] < 0)                                     ' reset flag?
      longfill(@Clock, 0, 6)                                    ' yes, clear everything
    waitcnt(t0 += ms1)                                          ' wait 1ms
    if (++Clock[REG_MS] == 1_000)                               ' one second?
      Clock[REG_MS]~                                            ' clear ms
      if (Clock[REG_SC] < posx)                                 ' if secs < max
        if ((++Clock[REG_SC] // 60) == 0)                       ' update secs, check for 1m
          if ((++Clock[REG_MN] // 60) == 0)                     ' update mins, check for 1hr
            if ((++Clock[REG_HR] // 24) == 0)                   ' update hrs, check for day
              ++Clock[REG_DY]                                   ' update hours 
      else
        Clock[REG_RST]~~                                        ' force reset
        
                        
DAT

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  